@@ -19,6 +19,9 @@
 """
 This module is responsible for all the things related to
 Barman configuration, such as parsing configuration file.
+
+:data COMPRESSIONS: A list of supported compression algorithms for WAL files.
+:data COMPRESSION_LEVELS: A list of supported compression levels for WAL files.
 """
 
 import collections
@@ -34,6 +37,7 @@ from glob import iglob
 from typing import List
 
 from barman import output, utils
+from barman.compression import compression_registry
 
 try:
     from ConfigParser import ConfigParser, NoOptionError
@@ -77,6 +81,15 @@ CREATE_SLOT_VALUES = ["manual", "auto"]
 # Config values relating to pg_basebackup compression
 BASEBACKUP_COMPRESSIONS = ["gzip", "lz4", "zstd", "none"]
 
+# WAL compression options
+COMPRESSIONS = compression_registry.keys()
+
+# WAL compression level options
+COMPRESSION_LEVELS = ["low", "medium", "high"]
+
+# Encryption options
+ENCRYPTION_VALUES = ["none", "gpg"]
+
 
 class CsvOption(set):
     """
@@ -344,6 +357,27 @@ def parse_backup_compression_location(value):
     raise ValueError("Invalid value (must be either `client` or `server`)")
 
 
+def parse_encryption(value):
+    """
+    Parse a string to valid encryption value.
+
+    Valid values are defined in :data:`ENCRYPTION_VALUES`.
+
+    :param str value: string value to be parsed
+    :raises ValueError: if the *value* is invalid
+    """
+    if value is not None:
+        value = value.lower()
+        if value == "none":
+            return None
+        if value not in ENCRYPTION_VALUES:
+            raise ValueError(
+                "Invalid encryption value '%s'. Allowed values are: %s."
+                % (value, ", ".join(ENCRYPTION_VALUES))
+            )
+    return value
+
+
 def parse_backup_method(value):
     """
     Parse a string to a valid backup_method value.
@@ -362,6 +396,49 @@ def parse_backup_method(value):
     )
 
 
+def parse_compression(value):
+    """
+    Parse a string to a valid compression option.
+
+    Valid values are the compression algorithms supported by Barman, as defined in
+    :data:`barman.compression.compression_registry`.
+
+    :param str value: compression value
+    :raises ValueError: if the value is invalid
+    """
+    if value:
+        value = value.lower()
+        if value not in COMPRESSIONS:
+            raise ValueError(
+                "Invalid value: '%s' (must be one in: %s)"
+                % (value, ", ".join(COMPRESSIONS))
+            )
+    return value
+
+
+def parse_compression_level(value):
+    """
+    Parse a string to a valid compression level option.
+
+    Valid values are ``low``, ``medium``, ``high`` and any integer number.
+
+    :param str value: compression_level value
+    :raises ValueError: if the value is invalid
+    """
+    if value:
+        value = value.lower()
+        # Handle negative compression levels
+        # Among the supported, only zstd allows negatives for now
+        if value.lstrip("-").isdigit():
+            value = int(value)
+        elif value not in COMPRESSION_LEVELS:
+            raise ValueError(
+                "Invalid value: '%s' (must be one in [%s] or an acceptable integer)"
+                % (value, ", ".join(COMPRESSION_LEVELS))
+            )
+    return value
+
+
 def parse_staging_path(value):
     if value is None or os.path.isabs(value):
         return value
@@ -516,12 +593,16 @@ class ServerConfig(BaseConfig):
         "check_timeout",
         "cluster",
         "compression",
+        "compression_level",
         "conninfo",
         "custom_compression_filter",
         "custom_decompression_filter",
         "custom_compression_magic",
         "description",
         "disabled",
+        "encryption",
+        "encryption_key_id",
+        "encryption_passphrase_command",
         "errors_directory",
         "forward_config_path",
         "gcp_project",
@@ -617,11 +698,15 @@ class ServerConfig(BaseConfig):
         "basebackup_retry_times",
         "check_timeout",
         "compression",
+        "compression_level",
         "configuration_files_directory",
         "create_slot",
         "custom_compression_filter",
         "custom_decompression_filter",
         "custom_compression_magic",
+        "encryption",
+        "encryption_key_id",
+        "encryption_passphrase_command",
         "forward_config_path",
         "gcp_project",
         "immediate_checkpoint",
@@ -690,7 +775,9 @@ class ServerConfig(BaseConfig):
         "basebackups_directory": "%(backup_directory)s/base",
         "check_timeout": "30",
         "cluster": "%(name)s",
+        "compression_level": "medium",
         "disabled": "false",
+        "encryption": "none",
         "errors_directory": "%(backup_directory)s/errors",
         "forward_config_path": "false",
         "immediate_checkpoint": "false",
@@ -739,7 +826,10 @@ class ServerConfig(BaseConfig):
         "basebackup_retry_sleep": int,
         "basebackup_retry_times": int,
         "check_timeout": int,
+        "compression": parse_compression,
+        "compression_level": parse_compression_level,
         "disabled": parse_boolean,
+        "encryption": parse_encryption,
         "forward_config_path": parse_boolean,
         "keepalive_interval": int,
         "immediate_checkpoint": parse_boolean,
